/*
 *         	  OMEGA IRC SECURITY SERVICES
 * 	      	    (C) 2008-2010 Omega Dev Team
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Desc: Module API demonstration 
 *
 *    $Id: m_example.c 1630 2010-01-01 08:50:44Z jer $
 *
 */

#include "stdinc.h"
#include "server.h"

/************************************************/
/**
 * Configurable Settings
 */
 
 
User* s_FooServ;
char *foo_nick = "FooServ";
char *ident = "security";
char *host  = "foos.dont.know";
char *modes = "+oSpq";
char *name  = "Foo Service";


/************************************************/
/**
 * Defines
 */

#define NAME "m_foo"
#define AUTHOR "Twitch"
#define VERSION "Module API Demonstration"

/************************************************/
/**
 * ProtoTypes
 */

static int foo_init		();
static void foo_connect ();

static void cmd_foo(User*,char*);

static void help_m_foo (User*);
static void help_foo_ext(User*);

static int Module_Init	();
static void Module_Close();
/**
 * The module header is required on all Modules it specifies the following
 * @param name 		(string)   - The module name
 * @param version 	(string)   - Current Module version
 * @param author 	(string)   - Author of the module
 * @param abi 		(MAKE_ABI) - The version of the binary interface that the module
 *								was designed for. This must be a MAKE_ABI returned value.
 * @param init_funct *(int)()  - Pointer to the Module factory.
 * @param close_funct *(void)() - Point to the closeing procedure.
 */
 

MODHEADER(NAME, VERSION, AUTHOR, MAKE_ABI(0,5,2),Module_Init, Module_Close);

/*****************************************/
/**
 *  Module_Init() - Module factory called
 *                when the module is loaded.
 *   return int Any status changes the module should pass to
 *				the core... see MOD API documentation.
 */

static int Module_Init()
{

      int status;
	  int ch;

      if (!(foo_init()))
				return MOD_STOP;
				
      ch = AddCmd(s_FooServ,"FOO", ACC_USER, cmd_foo); //add the command fo to the stack
	  AddHelpExt(ch,s_FooServ,help_foo_ext); //add command specific help using return from AddCmd
	  
	  AddHelp(s_FooServ,help_m_foo); //add standard help text.
      foo_connect();

	return MOD_CONT;
}

/**
 * Closing proceedure this is were you remove your clients
 * and commands and is called on unload.
 */
 
static void Module_Close()
{
	if (s_FooServ) {
		DelHelp(s_FooServ,help_m_foo); //delete the help from the bot
		DelClient(s_FooServ->nick); //delete our cilent
	}
	return;
}



/*************************************************/
/**
 * This function is a standard C style function
 * it returns true if our new client is created 
 */
 
static int foo_init()
{


    User* u;
	if ((s_FooServ = NewClient(foo_nick, ident, host, name))) //create a new client structure for our client
		return 1; //return true

       return 0; //we couldnt create the structure so bail.
}

/*************************************************/
/**
 * Introduce the client to server and put him nito the
 * log channel.
 */
 
static void foo_connect()
{
       Channel* c;
 	
	ircd_add_user (s_FooServ); //introduce our client
 	if (!(c = find_channel (logchan))) //check to see if log channel exists
   	{
			 c               = new_chan (logchan); //create log channel
          	 c->channelts    = time(NULL); //set internal channel ts to current time stamp.
    	}
	AddToChannelU (s_FooServ, c); //add our client to the channel. since we know it exists
	return;

}


/*****************************************/
/**
 * This is a command hook function
 * it is called when the command it is hooking 
 * is triggured.
 * @param u  (User *) - Pointer to the user structure of the person calling the command
 * @param av (string)	 - Paramaters passed after the command.
 * @return void
 */
 
static void cmd_foo(User *u, char* av)
{
	sendto_one(s_FooServ,u,"Your a fooo....!");
	return;
}

/*****************************************/

static void help_m_foo(User *u)
{
	sendto_one(s_FooServ,u,"FOO		- Foo Serv Help");
	return;
}


static void help_foo_ext(User *u)
{
		sendto_one(s_FooServ,u,"FOO");
		sendto_one(s_FooServ,u,"Module API Command Demonstration.");
		return;
}

